\name{gs2020}
\alias{gs2020}
\docType{data}
\title{Simulated Gsynth-like Panel Data (No Reversal)}
\description{A simulated dataset for demonstration and testing.}
\format{data.frame}
\keyword{datasets}
