\encoding{UTF-8}
\name{fect-internal}
\alias{XXinv}
\alias{beta_iter}
\alias{beta_iter_ub}
\alias{data_ub_adj}
\alias{Y_demean}
\alias{fe_add}
\alias{fe_add2}
\alias{fe_ad_iter}
\alias{fe_ad_covar_iter}
\alias{fe_ad_inter_iter}
\alias{fe_ad_inter_covar_iter}
\alias{_gsynth_XXinv}
\alias{_gsynth_beta_iter}
\alias{_gsynth_beta_iter_ub}
\alias{_gsynth_inter_fe}
\alias{_gsynth_inter_fe_ub}
\alias{_gsynth_inter_fe_mc}
\alias{_gsynth_data_ub_adj}
\alias{_gsynth_panel_beta}
\alias{_gsynth_panel_est}
\alias{_gsynth_panel_factor}
\alias{_gsynth_panel_factor_ub}
\alias{_gsynth_panel_fe}
\alias{_gsynth_panel_fe_ub}
\alias{_gsynth_Y_demean}
\alias{_gsynth_fe_add}
\alias{_gsynth_fe_add2}
\alias{_gsynth_fe_ad_iter}
\alias{_gsynth_fe_ad_covar_iter}
\alias{_gsynth_fe_ad_inter_iter}
\alias{_gsynth_fe_ad_inter_covar_iter}
\alias{inter_fe}
\alias{inter_fe_ub}
\alias{inter_fe_mc}
\alias{panel_beta}
\alias{panel_est}
\alias{panel_factor}
\alias{panel_factor_ub}
\alias{panel_fe}
\alias{panel_fe_ub}
\alias{fect.default}
\alias{fect.formula}
\alias{fect_fe}
\alias{fect_mc}
\alias{interFE.default}
\alias{interFE.formula}
\alias{cv.sample}
\alias{equiv_test}
\title{Internal FEct Functions}
\description{
 Internal fect functions
}
\details{
These are not to be called by the user (or in some cases are just
waiting for proper documentation to be written :).
}
\keyword{ts}
