\encoding{UTF-8}
\name{sim_gsynth}
\docType{data}
\alias{sim_gsynth}
\title{Simulated data for Gsynth (no treatment reversal)}
\description{A simulated dataset with no treatment reversal, used to demonstrate the generalized synthetic control method (never-treated estimation regime).}
\format{dataframe}
\keyword{datasets}
\references{
  Xu, Y. (2017).
  Generalized Synthetic Control Method: Causal Inference with Interactive Fixed Effects Models.
  \emph{Political Analysis, 25}(1), 57-76.
}
