\encoding{UTF-8}
\name{simgsynth}
\docType{data}
\alias{simgsynth}
\title{Simulated data for Gsynth}
\description{A simulated dataset with no treatment reversal.}
\format{dataframe}
\keyword{datasets}
\references{
  Xu, Y. (2017). 
  Generalized Synthetic Control Method: Causal Inference with Interactive Fixed Effects Models. 
  \emph{Political Analysis, 25}(1), 57-76.
}