\encoding{UTF-8}
\name{sim_base}
\docType{data}
\alias{sim_base}
\title{Simulated data (no interactive fixed effects)}
\description{A simulated dataset with continuous outcomes and no interactive fixed effects (factor loadings set to zero). Same DGP as \code{simdata} but with the factor contribution removed from the outcome.}
\format{dataframe}
\keyword{datasets}
\references{
  Liu, L., Wang, Y., and Xu, Y. (2022).
  A Practical Guide to Counterfactual Estimators for Causal Inference with Time-Series Cross-Sectional Data.
  \emph{American Journal of Political Science, 68}(1), 160-176.
}
