\encoding{UTF-8}
\name{turnout}
\docType{data}
\alias{turnout}
\title{EDR and Voter Turnout in the US}
\description{State-level voter turnout data.}
\format{dataframe}
\keyword{datasets}
\references{
  Springer, M. J. (2014). 
  How the States Shaped the Nation: American Electoral Institutions and Voter Turnout, 1920-2000. 
  University of Chicago Press.
  
  Xu, Y. (2017). 
  Generalized Synthetic Control Method: Causal Inference with Interactive Fixed Effects Models. 
  \emph{Political Analysis, 25}(1), 57-76.
}